@@ -5,23 +5,24 @@
 #' according to Abadie & L'Hour (2021). This function deals with only a
 #' single treated unit.
 #'
-#' @param X1 `N_covars by 1 matrix` of treated unit covariates
+#' @param X1 `N_covars by N_treated matrix` of treated unit covariates
 #' @param X0 `N_covars by N_donors matrix` of donor unit covariates
 #' @param v `N_covars vector` of variable weights (default 1)
 #' @param lambda `numeric` penalization parameter
 #' @param opt_pars `clarabel` settings using [clarabel::clarabel_control()]
 #' @param standardize `boolean` whether to standardize the input matrices (default TRUE)
+#' @param verbose `boolean` whether to print progress messages. Default on if in an interactive session.
 #'
 #' @details This routine uses the same notation of the original [Synth::synth()] implementation
 #' but uses a different, faster quadratic program solver (namely, [clarabel::clarabel()]).
 #' Additionally, it implements the penalization procedure described in Abadie & L'Hour (2021),
-#' such that the loss function is as in equation 5 of that paper (but for a single treated unit).
+#' such that the loss function is as in equation 5 of that paper.
 #'
 #' Variable weights are not optimized by this function, meaning they need to be pre-specified.
 #' This is by design.
 #'
 #' The original synthetic control method can be recovered by setting lambda = 0. For determining
-#' lambda based on data, see [cv_pensynth()].
+#' lambda based on hold-out data, see [cv_pensynth()].
 #'
 #' @references Abadie, A., & L’Hour, J. (2021).
 #' A penalized synthetic control estimator for disaggregated data.
@@ -34,10 +35,42 @@
 #'
 #' @example R/examples/example_pensynth.R
 #'
-#' @seealso [cv_pensynth()], [placebo_test()], [simulate_data()], [Synth::synth()]
+#' @seealso [cv_pensynth()], [placebo_test()], [simulate_data_synth()], [Synth::synth()]
 #'
 #' @export
-pensynth <- function(X1, X0, v = 1, lambda = 0, opt_pars = clarabel::clarabel_control(), standardize = TRUE) {
+pensynth <- function(
+  X1,
+  X0,
+  v = 1,
+  lambda = 0,
+  opt_pars = clarabel::clarabel_control(),
+  standardize = TRUE,
+  verbose = interactive()
+) {
+
+  N_treated <- if (is.matrix(X1)) ncol(X1) else 1
+  if (N_treated > 1) {
+    if (verbose) cli::cli_alert_info("Multiple treated units detected, estimating multiple models")
+
+    # a little recursion :)
+    results <- vector("list", N_treated)
+    if (verbose) cli::cli_progress_bar("Fitting models.")
+    for (n in 1:N_treated) {
+      results[[n]] <- pensynth(X1[,n,drop=FALSE], X0, v, lambda, opt_pars, standardize, verbose = FALSE)
+      if (verbose) cli::cli_progress_update()
+    }
+
+    return(structure(
+      .Data = list(
+        w = sapply(results, \(x) x[["w"]]),
+        solution = lapply(results, \(x) x[["solution"]]),
+        call = match.call()
+      ),
+      class = "pensynth"
+    ))
+  }
+
+  if (verbose) cli::cli_progress_step("Preparing data.")
   if (standardize) {
     st <- standardize_X(X1, X0)
     X0 <- st$X0
@@ -70,6 +103,7 @@ pensynth <- function(X1, X0, v = 1, lambda = 0, opt_pars = clarabel::clarabel_co
   )
 
   # Run the quadratic program solver
+  if (verbose) cli::cli_progress_step("Fitting model.")
   result <- clarabel::clarabel(
     P = X0VX0,
     q = -X1VX0 + lambda*Delta,
